import greenfoot.Actor;
import greenfoot.GreenfootImage;
import greenfoot.Greenfoot;
import greenfoot.MouseInfo;
import java.awt.Point;

/**
 * HomeBoard
 * 
 * @author Taylor Born
 * @version December 2013
 */
public class HomeBoard extends Board
{
    public static final int MAX_SHIPS = 5;
    private static final int[] SHIP_SIZES = { 5, 4, 3, 3, 2 };

    private Ship[] ships = new Ship[MAX_SHIPS];
    private boolean placing = true;

    public void finishPlacing()
    {
        placing = false;
    }
    
    /**
     * @return Null if a miss. Otherwise, the Boolean value indicates if sunk a ship.
     */
    public Boolean attack(int column, int row)
    {
        Ship ship = getShip(column, row);
        boolean hit = ship != null;
        newAttack(column, row, hit);
        if (hit)
            return new Boolean(ship.hit());
        return null;
    }
    
    public String getRemainingShipsReport()
    {
        String report = "";
        for (int r = 0; r < SIZE; r++)
            for (int c = 0; c < SIZE; c++)
                if (attacks[r][c] == null && getShip(c, r) != null)
                    report += (char)('A' + c) + "" + (char)('A' + r);
        return report;
    }
    
    private Ship getShip(int column, int row)
    {
        for (int i = 0; i < MAX_SHIPS; i++)
            if (ships[i].contains(column, row))
                return ships[i];
        return null;
    }
    
    private void initializeShipAt(Ship ship, int x, int y)
    {
        getWorld().addObject(ship, x, y);
    }
    
    private int getBoardX()
    {
        return getX();
    }
    private int getBoardY()
    {
        return getY();
    }
    
    @Override
    public void reset()
    {
        super.reset();
        placing = true;
        for (int i = 0; i < MAX_SHIPS; i++)
        {
            getWorld().removeObject(ships[i]);
            ships[i] = new Ship(i);
        }
    }
    
    private class Ship extends Actor
    {
        private Point[] ends = new Point[2];
        private int size;
        private int health;
        
        public Ship(int i)
        {
            size = SHIP_SIZES[i];
            
            int x = getXFor(i);
            int y = (getYFor(0) + getYFor(size - 1)) / 2;
            initializeShipAt(this, x, y);
            
            ends[0] = new Point(i, 0);
            ends[1] = new Point(i, size - 1);
            
            health = size;
            setImage(new GreenfootImage(i + ".png"));
            setRotation(90);
        }
        private Point pressedAt;
        
        @Override
        public void act()
        {
            if (!placing)
            {
                if (pressedAt != null)
                {
                    relocate(getRotation() == 90);
                    pressedAt = null;
                }
                return;
            }
            MouseInfo mouse = Greenfoot.getMouseInfo();
            if (Greenfoot.mousePressed(this))
            {
                int x = getX();
                int y = getY();
                getWorld().removeObject(this);
                initializeShipAt(this, x, y);
                
                pressedAt = new Point(mouse.getX() - x, mouse.getY() - y);
            }
            
            if (pressedAt != null && (Greenfoot.mouseClicked(null) || Greenfoot.mouseDragEnded(null)))
            {
                boolean vertical = getRotation() == 90;
                
                // Position of middle of first cell of this Ship's image
                int x = getX();
                int y = getY();
                if (vertical)
                    y -= size * CELL_SIZE / 2 - CELL_SIZE / 2;
                else
                    x -= size * CELL_SIZE / 2 - CELL_SIZE / 2;
                // Translate values to be in relation to the board
                x -= getBoardX() - WIDTH / 2;
                y -= getBoardY() - WIDTH / 2;
                // Translate values to column and row
                x /= CELL_SIZE;
                y /= CELL_SIZE;
                
                if (x >= 0 && x < SIZE - (vertical ? 0 : size - 1) && y >= 0 && y < SIZE - (vertical ? size - 1 : 0) && fitsAt(x, y, vertical))
                    set(x, y, vertical);
                else
                    relocate(vertical);
                
                pressedAt = null;
            }
            if (pressedAt != null)
            {
                if (Greenfoot.mouseDragged(null))
                    setLocation(mouse.getX() - (int)pressedAt.getX(), mouse.getY() - (int)pressedAt.getY());
                if (KeysPressed.check("r"))
                {
                    pressedAt = new Point(0, 0);
                    setRotation(90 - getRotation());
                    setLocation(mouse.getX(), mouse.getY());
                }
            }
        }
        
        private boolean fitsAt(int column, int row, boolean vertical)
        {
            for (int i = 0; i < size; i++)
            {
                int c = column + (vertical ? 0 : i);
                int r = row + (vertical ? i : 0);
                if (c < 0 || c >= SIZE || r < 0 || r >= SIZE || getShip(c, r) != null)
                    return false;
            }
            return true;
        }
        private void relocate(boolean vertical)
        {
            for (int r = 0; r < SIZE; r++)
                for (int c = 0; c < SIZE; c++)
                    if (fitsAt(c, r, vertical))
                    {
                        set(c, r, vertical);
                        return;
                    }
        }
        private void set(int column, int row, boolean vertical)
        {
            ends[0] = new Point(column, row);
            if (vertical)
            {
                ends[1] = new Point(column, row + size - 1);
                setLocation(getXFor(column), (getYFor(row) + getYFor(row + size - 1)) / 2);
            }
            else
            {
                ends[1] = new Point(column + size - 1, row);
                setLocation((getXFor(column) + getXFor(column + size - 1)) / 2, getYFor(row));
            }
        }
        
        public boolean hit()
        {
            return --health == 0;
        }
        public Point[] getEnds()
        {
            return ends;
        }
        public boolean contains(int column, int row)
        {
            if (pressedAt != null)
                return false;
            if (ends[0].getX() == ends[1].getX())
                return column == ends[0].getX() && row >= ends[0].getY() && row <= ends[1].getY();
            return row == ends[0].getY() && column >= ends[0].getX() && column <= ends[1].getX();
        }
    }
}